package cn.com.ty.lift.ud.chat.service.impl;

import java.time.LocalDateTime;
import java.util.List;

import cn.com.ty.lift.common.model.PushMessage;
import cn.com.ty.lift.common.model.PushUserInfo;
import cn.com.ty.lift.ud.chat.controller.model.ChatSessionReq;
import cn.com.ty.lift.ud.chat.controller.model.ChatSessionVo;
import cn.com.ty.lift.ud.chat.controller.query.ChatSessionQuery;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.core.JmsMessagingTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.com.ty.lift.ud.chat.mapper.ChatSessionMapper;
import cn.com.ty.lift.ud.chat.mapper.entity.ChatSessionEntity;
import cn.com.ty.lift.ud.chat.service.IChatSessionService;
import cn.com.ty.lift.ud.userAccount.service.IUserAccountService;
import cn.com.ty.lift.ud.userInfo.mapper.UserInfoMapper;
import cn.com.ty.lift.ud.userInfo.mapper.entity.UserInfoEntity;
import cn.com.ty.lift.ud.utils.UUIDUtil;
import cn.com.xwy.boot.web.dto.RestResponse;

/**
 * 会话 -ServiceImpl
 *
 * @author Xson
 * @date 2020-04-23
 */
@Service
@Transactional
public class ChatSessionServiceImpl extends ServiceImpl<ChatSessionMapper, ChatSessionEntity>
    implements IChatSessionService {

    private @Autowired ChatSessionMapper sessionMapper;
    
    private @Autowired IUserAccountService iUserAccountService;
    
    private @Autowired UserInfoMapper userInfoMapper;
    
    @Autowired
    private JmsMessagingTemplate jmsMessagingTemplate;

    @Override
    public ChatSessionEntity findOneBySessionIdAndPwd(String sessionId, String pwd) {
        return sessionMapper.findOneBySessionIdAndPwd(sessionId, pwd);
    }

    @Override
    public RestResponse existRoom(String room) {

        QueryWrapper<ChatSessionEntity> query = new QueryWrapper<ChatSessionEntity>();
        query.eq("sessionid", room);
        query.eq("statuz", 1);

        boolean flag = sessionMapper.selectOne(query) == null ? false : true;
        if (flag)
            return RestResponse.success(flag, "successfully");
        else
            return RestResponse.success(flag, "房间已关闭，或未找到");
    }

    @Override
    public RestResponse batchUpdate(ChatSessionReq req) {

        int flag = sessionMapper.batchUpdate(req);
        if (flag > 0)
            return RestResponse.success("Updated successfully");
        else
            return RestResponse.fail("Updated unsuccessfully");
    }

    @Override
    public RestResponse selectByExample(ChatSessionQuery query) {
        IPage page = query.page();
        List<ChatSessionEntity> list = sessionMapper.selectByQuery(query.condition(), query);
        int total = sessionMapper.countByQuery(query.condition());
        page.setRecords(list);
        page.setTotal(total);
        return RestResponse.success(page);
    }

    @Override
    public int insertSelective(ChatSessionEntity record) {
        return sessionMapper.insert(record);
    }

    @Override
    public RestResponse findOneByLiftCase(ChatSessionQuery req) {
        QueryWrapper<ChatSessionEntity> queryWrapper = new QueryWrapper<ChatSessionEntity>();
        queryWrapper.eq("data_id", req.getDataId());
        queryWrapper.eq("user_id", req.getUserId());
        ChatSessionEntity data = sessionMapper.selectOne(queryWrapper);

        return RestResponse.success(data, "success");
    }

    @Override
    public List<String> findListByUserId(String userId) {
        return sessionMapper.findListByUserId(userId);
    }

    @Override
    public RestResponse batchUpdateEntrance(ChatSessionReq req) {
        sessionMapper.batchUpdateEntrance(req.getSessionId(), 0);
        return RestResponse.success();
    }

    @Override
    public RestResponse findDBListByUserId(ChatSessionQuery query) {
        IPage page = query.page();
        List<ChatSessionEntity> list = sessionMapper.findDBListByUserId(query.getUserId());
        page.setRecords(list);
        return RestResponse.success(page);
    }

    @Override
    public RestResponse createPrivateChatRoom(ChatSessionQuery request) {
        ChatSessionVo vo = new ChatSessionVo();
        // 使用我的用户id和将要聊天的用户id正反去数据库查询房间号是否存在
        String sessionId =
            sessionMapper.findOneByUserIdAndCreateUserIdAndDateTable(request.getUserId(), request.getCreateUserId(), 3);
        if (StringUtils.isBlank(sessionId)) {
            sessionId = UUIDUtil.getUuidByTime9();
            ChatSessionEntity record = new ChatSessionEntity();
            record.setUserId(request.getUserId());
            record.setCreateUserId(request.getCreateUserId());
            record.setSessionid(sessionId);
            record.setDataTable(3);
            record.setEntrance(1);
            record.setStatuz(1);
            record.setCreateTime(LocalDateTime.now());
            sessionMapper.insertSelective(record);
            
            record = new ChatSessionEntity();
            record.setUserId(request.getCreateUserId());
            record.setCreateUserId(request.getUserId());
            record.setSessionid(sessionId);
            record.setDataTable(3);
            record.setEntrance(1);
            record.setStatuz(1);
            record.setCreateTime(LocalDateTime.now());
            sessionMapper.insertSelective(record);
            
            vo.setType(1);
            vo.setRoom(sessionId);
        } else {
            vo.setType(2);
            vo.setRoom(sessionId);
        }
        
     // 推送消息
        UserInfoEntity byUserId = userInfoMapper.getByUserId(Long.parseLong(request.getUserId()));
        PushUserInfo pushUserInfo =
            iUserAccountService.getPushUserInfoByUserId(request.getCreateUserId().toString());
        PushMessage pushMessage = PushMessage.sayHelloToFriend(byUserId.getName());
        pushMessage.sendTokenOnPlatform(jmsMessagingTemplate, pushUserInfo);
        
        return RestResponse.success(vo);
    }

}
