package cn.com.ty.lift.ud.neety.task;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.boot.CommandLineRunner;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import cn.com.ty.lift.ud.utils.ApplicationContextUtil;
import cn.com.ty.lift.ud.neety.protocol.RoomVo;
import cn.com.ty.lift.ud.redis.RedisUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 更新缓存中用户在线数据状态
 * @author sKing
 * @Date 2020年6月8日
 */
@Slf4j
@Component
@Lazy
public class RedisRunner implements CommandLineRunner {

    private static RedisUtil redis = ApplicationContextUtil.getBean(RedisUtil.class);

    @Override
    public void run(String... args) throws Exception {
        log.info("项目启动成功，更新redis中用户在线状态, start!");

        Map<String, List<RoomVo>> redisRoom = redis.get("ROOM", Map.class);
        if (null != redisRoom) {

            for (String room : redisRoom.keySet()) {
                String txt = redis.getKey("ROOM");
                Object ob = txt == null ? null : JSONObject.parseObject(txt, HashMap.class).get(room);
                if (null != ob) {
                    List<RoomVo> voList = null;
                    try {
                        voList = JSONArray.parseArray(ob.toString(), RoomVo.class);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                    if (voList != null && !voList.isEmpty()){
                        for (RoomVo item : voList) {
                            item.setFlag(false);
                        }
                        redisRoom.put(room, voList);
                    }
                }
            }

            // 重置缓存
            redis.setValue("ROOM", redisRoom);
            log.info("更新redis缓存 end!");
        }

    }

}
